@@ -14,6 +14,8 @@ A user interface for Lino applications that uses FaceBooks React JS framework.
     models
 """
 
+from django.utils import translation
+from lino.core import constants
 from lino.api.ad import Plugin
 
 
@@ -29,6 +31,7 @@ class Plugin(Plugin):
 
     media_name = 'react'
     support_async = True
+    top_paginator = True
 
     # media_base_url = "http://maxcdn.bootstrapcdn.com/bootstrap/3.2.0/"
 
@@ -36,7 +39,28 @@ class Plugin(Plugin):
         from .renderer import Renderer
         self.renderer = Renderer(self)
         # ui.bs3_renderer = self.renderer
-        kernel.extjs_renderer = self.renderer
+        # removed 20230613 kernel.extjs_renderer = self.renderer
+
+    def load_site_js_snippets(self, settings):
+        context = dict(
+            extjs_renderer=self.renderer,
+            site=self.site,
+            settings=settings,
+            lino=self.site.plugins.lino,
+            language=translation.get_language(),
+            constants=constants,
+            extjs=None,  # 20230613 used in sepa/config/iban/uppercasetextfield.js
+        )
+
+        js = ""
+        env = self.site.plugins.jinja.renderer.jinja_env
+
+        for p in self.site.sorted_plugins:
+            for snippet in p.site_js_snippets:
+                template = env.get_template(snippet)
+                js += f"<script>{template.render(**context)}</script>"
+
+        return js
 
     def get_patterns(self):
         # this is called once after startup.
