@@ -1,8 +1,8 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2012-2022 Rumma & Ko Ltd
+# Copyright 2012-2023 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
 
-
+import json
 import os
 from pathlib import Path
 from html import escape
@@ -11,9 +11,7 @@ from django.db import models
 from django.utils.text import format_lazy
 from django.utils import translation
 
-from lino.core import constants as ext_requests
-from lino.core.renderer import add_user_language, JsRenderer, HtmlRenderer
-from lino.core.renderer_mixins import JsCacheRenderer
+from lino.core.renderer import add_user_language, JsCacheRenderer
 
 from lino.core.menus import Menu, MenuItem
 from lino.core import constants
@@ -39,6 +37,7 @@ from lino.utils import jsgen
 from lino.utils.jsgen import py2js, js_code, obj2dict
 
 from lino.modlib.users.utils import get_user_profile, with_user_profile
+from lino.modlib.system.mixins import EditSafe
 
 from inspect import isclass
 
@@ -58,7 +57,7 @@ def find(itter, target, key=None):
         return -1
 
 
-class Renderer(JsRenderer, JsCacheRenderer):
+class Renderer(JsCacheRenderer):
     """
     A front-end renderer that uses the React Javascript framework.
     """
@@ -70,8 +69,7 @@ class Renderer(JsRenderer, JsCacheRenderer):
     support_dashboard_layout = True  # Used to prevent any dashboard items being rendered from main html dashboard.
 
     def __init__(self, front_end):
-        JsRenderer.__init__(self, front_end)
-        JsCacheRenderer.__init__(self)
+        super().__init__(front_end)
         jsgen.register_converter(self.py2js_converter)
 
     def write_lino_js(self, f):
@@ -97,11 +95,14 @@ class Renderer(JsRenderer, JsCacheRenderer):
         plugins = settings.SITE.plugins
         data = dict(
             actions={a.action_name: self.action2json(a) for a in actions},
+            editing_frontend=self.front_end.url_prefix is not None,
             # actors={a.actor_id: a for a in self.actors_list},
+            languages={lang.django_code: lang.name for lang in settings.SITE.languages},
             menu=settings.SITE.get_site_menu(get_user_profile()),
-            use_dashboard_layouts=plugins.system.use_dashboard_layouts,
             site_title=settings.SITE.title,
-            )
+            top_paginator=self.front_end.top_paginator,
+            use_dashboard_layouts=plugins.system.use_dashboard_layouts,
+        )
         if settings.SITE.is_installed('memo'):
             # [#,@] key triggers
             data.update(
@@ -112,7 +113,9 @@ class Renderer(JsRenderer, JsCacheRenderer):
         # else:
         data.update(revision_number=settings.SITE.kernel.code_mtime)
         try:
-            data.update(mjs_timestamp=os.stat(Path(settings.STATIC_ROOT) / 'react' / 'main.js').st_mtime)
+            data[constants.URL_PARAM_MAIN_JS_TIMESTAMP] = os.stat(
+                Path(settings.STATIC_ROOT) / 'react' / 'main.js'
+            ).st_mtime
         except FileNotFoundError:
             pass
 
@@ -152,7 +155,8 @@ class Renderer(JsRenderer, JsCacheRenderer):
 
         # print("20210417", data)
 
-        f.write(py2js(data))
+        json.dump(json.loads(py2js(data)), f, indent=4)
+        # f.write(py2js(data))
         # f.write(py2js(data, compact=not settings.SITE.is_demo_site))
         self.serialise_js_code = False
         return 1
@@ -198,6 +202,7 @@ class Renderer(JsRenderer, JsCacheRenderer):
         assert isinstance(v, LayoutElement)
         # Layout elems
         result = dict(label=v.get_label(),
+                      sortable=v.sortable,
                       # repr=repr(v),
                       react_name=v.__class__.__name__)  # Used for choosing correct react component
         if hasattr(v, "elements"):  # dd
@@ -237,15 +242,15 @@ class Renderer(JsRenderer, JsCacheRenderer):
         kw.update(st['base_params'])
         add_user_language(kw, ar)
         if ar.offset is not None:
-            kw.setdefault(ext_requests.URL_PARAM_START, ar.offset)
+            kw.setdefault(constants.URL_PARAM_START, ar.offset)
         if ar.limit is not None:
-            kw.setdefault(ext_requests.URL_PARAM_LIMIT, ar.limit)
+            kw.setdefault(constants.URL_PARAM_LIMIT, ar.limit)
         if ar.order_by is not None:
             sc = ar.order_by[0]
             if sc.startswith('-'):
                 sc = sc[1:]
-                kw.setdefault(ext_requests.URL_PARAM_SORTDIR, 'DESC')
-            kw.setdefault(ext_requests.URL_PARAM_SORT, sc)
+                kw.setdefault(constants.URL_PARAM_SORTDIR, 'DESC')
+            kw.setdefault(constants.URL_PARAM_SORT, sc)
         # ~ print '20120901 TODO get_request_url
 
         return self.front_end.build_plain_url(
@@ -447,21 +452,28 @@ class Renderer(JsRenderer, JsCacheRenderer):
 
         if not status:
             status = {}
-        if ar and ar._status:
-            status.update(ar._status)
+
+        if ar is not None and hasattr(ar, 'get_status'):
+            status = ar.get_status(**status)
 
         if ar and ar.subst_user:
             status[constants.URL_PARAM_SUBST_USER] = ar.subst_user
-        if isinstance(a, ShowEmptyTable):
-            status.update(record_id=-99998)
-        if ar and ar.master_instance is not None:
-            mi = ar.master_instance
-            bp = {
-                constants.URL_PARAM_MASTER_PK: mi.pk}
-            if issubclass(mi.__class__, models.Model):
-                mt = ContentType.objects.get_for_model(mi.__class__)
-                bp[constants.URL_PARAM_MASTER_TYPE] = mt
-            status.update(base_params=bp)
+
+        # if ar and ar._status:
+        #     status.update(ar._status)
+        #
+        # if ar and ar.subst_user:
+        #     status[constants.URL_PARAM_SUBST_USER] = ar.subst_user
+        # if isinstance(a, ShowEmptyTable):
+        #     status.update(record_id=-99998)
+        # if ar and ar.master_instance is not None:
+        #     mi = ar.master_instance
+        #     bp = {
+        #         constants.URL_PARAM_MASTER_PK: mi.pk}
+        #     if issubclass(mi.__class__, models.Model):
+        #         mt = ContentType.objects.get_for_model(mi.__class__)
+        #         bp[constants.URL_PARAM_MASTER_TYPE] = mt
+        #     status.update(base_params=bp)
         if an == 'grid' and hasattr(status, 'record_id'):
             del status['record_id']
 
@@ -519,7 +531,7 @@ class Renderer(JsRenderer, JsCacheRenderer):
             lh = ba.get_layout_handel()
             if (lh or ba.action.window_type):
 
-                action_descriptor = {'an': ba.action.action_name}
+                action_descriptor = { constants.URL_PARAM_ACTION_NAME: ba.action.action_name}
                 if lh:
                     action_descriptor.update(window_layout=lh.layout._formpanel_name)
                 if ba.action.window_type:
@@ -644,6 +656,9 @@ class Renderer(JsRenderer, JsCacheRenderer):
         if hasattr(v.model, "file"):
             result.update(contain_media=True)
 
+        if isclass(v.model) and issubclass(v.model, EditSafe):
+            result.update(edit_safe=True)
+
         if v.display_mode is not None:
             result['display_mode'] = self.display_mode2json(v.display_mode)
 
@@ -670,10 +685,11 @@ class Renderer(JsRenderer, JsCacheRenderer):
             result.update(chooser_dict={fn: [cf.name for cf in c.context_fields]
                                         for fn, c in chooser_dict.items()})
 
-        if settings.SITE.is_installed('contenttypes') and getattr(v, 'model', None) is not None and hasattr(v.model,
-                                                                                                            "_meta"):
+        if settings.SITE.is_installed('contenttypes') and hasattr(v.model, "_meta"):
+            # and getattr(v, 'model', None) is not None
             # Perhaps I should have the model also be py2js'd?
-            result.update(content_type=ContentType.objects.get_for_model(v.model).pk)
+            if not v.model._meta.abstract:
+                result.update(content_type=ContentType.objects.get_for_model(v.model).pk)
         for k in ("detail_action", "insert_action", "default_action"):
             ba = getattr(v, k, None)
             if ba is not None:
@@ -699,7 +715,8 @@ class Renderer(JsRenderer, JsCacheRenderer):
             fn = os.path.join(*self.lino_js_parts_chunked(actor.actor_id))
 
             def write(f):
-                f.write(py2js(self.actor2json(actor)))
+                # f.write(py2js(self.actor2json(actor)))
+                json.dump(json.loads(py2js(self.actor2json(actor))), f, indent=4)
 
             settings.SITE.kernel.make_cache_file(fn, write, force)
 
